from pathlib import Path

# Input file character set
ENCODING = 'utf-8'

# Default language code
# https://en.wikipedia.org/wiki/ISO_639-1
LANGUAGE = 'en'

FONT_FAMILY = 'DejaVu Sans'
FONT_SIZE = 11
FONT = Path('./font/DejaVuSans.ttf')
UNIT = 'mm'
IMAGE_WIDTH = 150
